package com.io.yutian.auchestshop.database;

import com.io.yutian.auchestshop.AuChestShop;
import com.io.yutian.auchestshop.shop.Shop;
import com.io.yutian.aulib.serialize.SerializeHelper;
import com.io.yutian.aulib.sql.SQLHelper;
import com.io.yutian.aulib.sql.api.SQLManager;
import com.io.yutian.aulib.sql.api.SQLQuery;
import com.io.yutian.aulib.sql.api.enums.IndexType;
import com.io.yutian.aulib.sql.manager.SQLManagerImpl;
import com.io.yutian.aulib.sql.util.DatabaseDriverType;
import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;
import org.json.JSONObject;

import java.io.File;
import java.io.IOException;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.List;

public class SQLIO {

    private SQLManager sqlManager;

    public void init() throws IOException {
        HikariConfig config = new HikariConfig();
        config.setDriverClassName(DatabaseDriverType.SQLITE.getDriverClassName());
        config.setJdbcUrl("jdbc:sqlite:"+new File(AuChestShop.inst().getDataFolder(), "shops.db").getCanonicalFile().getAbsolutePath());
        sqlManager = new SQLManagerImpl(new HikariDataSource(config), "AuChestShop-Hikari-SQLManager");
        sqlManager.setExecutorPool(SQLExecutor.HIKARICP_EXECUTOR);

        createTables();
    }

    private void createTables() {
        sqlManager.createTable("shops")
                .addColumn("uuid", "VARCHAR(48) NOT NULL")
                .addColumn("owner", "VARCHAR(48) NOT NULL")
                .addColumn("data", "TEXT NOT NULL")
                .setIndex("uuid", IndexType.PRIMARY_KEY)
                .setTableSettings("")
                .build().execute(null);
    }

    public List<Shop> loadAllShops() {
        List<Shop> shops = new ArrayList<>();
        sqlManager.createQuery()
                .inTable("shops")
                .build().executeAsync(sqlQuery -> {
                    ResultSet resultSet = sqlQuery.getResultSet();
                    while (resultSet.next()) {
                        String uuid = resultSet.getString("uuid");
                        String owner = resultSet.getString("owner");
                        String data = resultSet.getString("data");
                        JSONObject dataJson = new JSONObject(data);
                        Shop shop = SerializeHelper.deserialize(Shop.class, dataJson);
                        shops.add(shop);
                    }
                });
        return shops;
    }

    public void saveAllShops(List<Shop> shops) {
        try {
            List<Shop> insertedShops = new ArrayList<>();
            for (Shop shop : shops) {
                try (SQLQuery sqlQuery = sqlManager.createQuery().inTable("shops").addCondition("uuid", shop.getUUID().toString()).setLimit(1).build().execute()) {
                    try (ResultSet resultSet = sqlQuery.getResultSet()) {
                        if (resultSet.next()) {
                            JSONObject dataJson = SerializeHelper.serialize(shop);
                            sqlManager.createUpdate("shops")
                                    .addCondition("uuid", shop.getUUID().toString())
                                    .setLimit(1)
                                    .setColumnValues("owner", shop.getOwner().toString())
                                    .setColumnValues("data", dataJson.toString())
                                    .build().execute();
                        } else {
                            insertedShops.add(shop);
                        }
                    }
                }
            }
            List<Object[]> params = new ArrayList<>();
            for (Shop shop : insertedShops) {
                JSONObject dataJson = SerializeHelper.serialize(shop);
                params.add(new Object[]{shop.getUUID().toString(), shop.getOwner().toString(), dataJson.toString()});
            }
            if (!params.isEmpty()) {
                sqlManager.createInsertBatch("shops").setIgnore(false).setColumnNames("uuid", "owner", "data").setAllParams(params).executeAsync();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void close() {
        if (sqlManager == null) {
            return;
        }
        SQLHelper.shutdownManager(sqlManager);
    }

}
