package com.io.yutian.auchestshop.manager;

import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import com.io.yutian.auchestshop.shop.Shop;
import com.io.yutian.auchestshop.shop.ShopChunk;
import com.io.yutian.auchestshop.shop.ShopLoader;
import com.io.yutian.auchestshop.shop.ShopLocation;
import com.io.yutian.auchestshop.util.ShopUtil;
import org.apache.commons.lang.StringUtils;
import org.bukkit.Chunk;
import org.bukkit.Location;
import org.bukkit.World;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.*;

public class ShopManager {

    private final Map<String, Map<ShopChunk, Map<Location, Shop>>> shops = Maps.newConcurrentMap();
    private final Set<Shop> loadedShops = Sets.newConcurrentHashSet();

    public void loadAll(ShopLoader shopLoader) {

    }

    public @NotNull List<Shop> getAllShops() {
        List<Shop> shopsCollected = new ArrayList<>();
        for (Map<ShopChunk, Map<Location, Shop>> shopMapData : getShops().values()) {
            for (Map<Location, Shop> shopData : shopMapData.values()) {
                shopsCollected.addAll(shopData.values());
            }
        }
        return shopsCollected;
    }

    public @NotNull List<Shop> getAllShops(@NotNull UUID playerUUID) {
        List<Shop> playerShops = new ArrayList<>();
        for (Shop shop : getAllShops()) {
            UUID shopUuid = shop.getOwner();
            if (playerUUID.equals(shopUuid)) {
                playerShops.add(shop);
            }
        }
        return playerShops;
    }


    public @Nullable Shop getShop(UUID shopId) {
        for (Shop shop : getAllShops()) {
            if (shop.getUUID().equals(shopId)) {
                return shop;
            }
        }
        return null;
    }

    public @Nullable Shop getShop(@NotNull Location location) {
        if (!ShopUtil.isShoppables(location.getBlock().getType())) {
            return null;
        }
        ShopChunk shopChunk = ShopChunk.fromLocation(location);
        final Map<Location, Shop> inChunk = getShops(shopChunk);
        if (inChunk == null) {
            return null;
        }
        location = location.clone();
        location.setX(location.getBlockX());
        location.setY(location.getBlockY());
        location.setZ(location.getBlockZ());
        return inChunk.get(location);
    }

    public Map<String, Map<ShopChunk, Map<Location, Shop>>> getShops() {
        return this.shops;
    }

    public Map<Location, Shop> getShops(@NotNull Chunk chunk) {
        return getShops(chunk.getWorld().getName(), chunk.getX(), chunk.getZ());
    }

    public Map<Location, Shop> getShops(@NotNull String world, int chunkX, int chunkZ) {
        final Map<ShopChunk, Map<Location, Shop>> inWorld = this.getShops(world);
        if (inWorld == null) {
            return null;
        }
        return inWorld.get(new ShopChunk(world, chunkX, chunkZ));
    }

    public Map<Location, Shop> getShops(@NotNull ShopChunk shopChunk) {
        return getShops(shopChunk.getWorld(), shopChunk.getX(), shopChunk.getZ());
    }

    public Map<ShopChunk, Map<Location, Shop>> getShops(@NotNull String world) {
        return this.shops.get(world);
    }

    public @NotNull List<Shop> getShopsInWorld(@NotNull World world) {
        final List<Shop> worldShops = new ArrayList<>();
        for (final Shop shop : getAllShops()) {
            ShopLocation location = shop.getLocation();
            if (Objects.equals(location.getWorld(), world)) {
                worldShops.add(shop);
            }
        }
        return worldShops;
    }

    public @NotNull List<Shop> getShopsInWorld(@NotNull String worldName) {
        final List<Shop> worldShops = new ArrayList<>();
        for (final Shop shop : getAllShops()) {
            ShopLocation location = shop.getLocation();
            if (StringUtils.equals(worldName, location.getWorld())) {
                worldShops.add(shop);
            }
        }
        return worldShops;
    }

    public void loadShop(@NotNull Shop shop) {
        loadedShops.add(shop);
    }

    public void unloadShop(@NotNull Shop shop) {
        this.loadedShops.remove(shop);
    }

    public Set<Shop> getLoadedShops() {
        return loadedShops;
    }

}
